classdef laplaceinv < handle
    %The class laplaceinv provides an opject for the computational solution
    %of 
    %
    % u = (I - delta * Delta)^-1*f s.t. Neumann boundary conditions
    %
    %with Delta denoting the Laplace operator.
    
    properties(SetAccess = public, GetAccess = public)
        delta %regularization parameter                
    end

    properties(SetAccess = private, GetAccess = public)
        kernel %representation of discrete Laplacian in DCT space
    end        
    
    methods
        function L = laplaceinv(sizevec, hvec)
            if nargin == 0
                sizevec = [256 256];                
            end
            if nargin <= 1
                hvec = ones(2, 1);
            end
            %initialize dim-dimensional grid for kernel computation
            if sizevec(2) == 1
                dim = 1;
            else
                dim = numel(sizevec);
            end
            p = (0:1:(sizevec(1) - 1))';
            for i=2:dim
                pnew = (0:1:(sizevec(i) - 1))';                
                K1 = kron(speye(i - 1), kron(ones([sizevec(i) 1]), ...
                    speye(prod(sizevec(1:(i - 1))))));
                K2 = kron(speye(sizevec(i)), ones(prod(sizevec(1:(i - ...
                    1))), 1));
                p = [K1*p; K2*pnew];                
            end            
            %set up dct kernel for inversion
            L.kernel = zeros([prod(sizevec), 1]);
            for i=1:dim
                L.kernel = L.kernel + 4*(sin(pi*p(((i - 1)*prod( ...
                    sizevec) + 1):(i*prod(sizevec)))/(2*sizevec(i)))/ ...
                    hvec(i)).^2;
            end
            L.kernel = reshape(L.kernel, sizevec);            
            L.delta = 1;
        end
        function u = mldivide(L, f)     
            if isequal(size(f), size(L.kernel));
                u = L.mirt_idctn(L.mirt_dctn(f)./(1 + L.delta * L.kernel));                        
            elseif isequal(numel(size(f)), numel(size(L.kernel)) + 1)
                noc = size(f, numel(size(f)));
                f = shiftdim(f, numel(size(L.kernel)));
                u = zeros(size(f));                               
                for i=1:noc
                    u(i, :) = reshape(L.mirt_idctn(L.mirt_dctn(reshape(...
                        f(i, :), size(L.kernel)))./(1 + L.delta * ...
                        L.kernel)), [1 numel(L.kernel)]);                    
                end
                u = shiftdim(u, 1);
            else
                error('Dimensions must agree!');
            end
        end
        function setalpha(L, val)
            L.delta = val;
        end
        function setdelta(L, val)
            L.delta = val;
        end        
        function setlambda(L, val)
            L.delta = 1/val;
        end
    end        
    
    methods (Static)
        a = mirt_dctn(a)
        a = mirt_idctn(a)
    end
    
end

